#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#define ARRAY_SIZE 10

int partition(int arr[], int low, int high);

void swap(int *a, int *b) {
    int temp = *a;
    *a = *b;
    *b = temp;
}

void qs_sort(int arr[], int low, int high) {
    if (low < high) {
        
        int pivot = partition(arr, low, high);
        
        qs_sort(arr, low, pivot - 1);
        qs_sort(arr, pivot + 1, high);
    }
}

int partition(int arr[], int low, int high) {
    int pivot = arr[high];
    int i = (low - 1);
    
    for (int j = low; j <= high - 1; j++) {
        if (arr[j] < pivot) {
            i++;
            swap(&arr[i], &arr[j]);
        }
    }
    swap(&arr[i + 1], &arr[high]);
    return (i + 1);
}

void insertion_sort(int arr[], int n) {
    int i, key, j;
    for (i = 1; i < n; i++) {
        key = arr[i];
        j = i - 1;
        
        while (j >= 0 && arr[j] > key) {
            arr[j + 1] = arr[j];
            j = j - 1;
        }
        arr[j + 1] = key;
    }
}

int is_sorted(int arr[], int n) {
    for (int i = 0; i < n - 1; i++) {
        if (arr[i] > arr[i + 1]) {
            return 0; 
        }
    }
    return 1; 
}

int arrays_equal(int arr1[], int arr2[], int n) {
    for (int i = 0; i < n; i++) {
        if (arr1[i] != arr2[i]) {
            return 0; 
        }
    }
    return 1;
}

void print_array(int arr[], int n) {
    for (int i = 0; i < n; i++) {
        printf("%d ", arr[i]);
    }
    printf("\n");
}

int main() {
    int arr1[ARRAY_SIZE], arr2[ARRAY_SIZE], arr3[ARRAY_SIZE];
    clock_t start, end;
    
    // Заповнення масивів випадковими числами
    srand(time(NULL));
    for (int i = 0; i < ARRAY_SIZE; i++) {
        arr1[i] = rand();
        arr2[i] = arr1[i];
        arr3[i] = arr1[i];
    }
    
    if (is_sorted(arr1, ARRAY_SIZE)) {
        printf("The array of random numbers is already sorted.\n");
    } else {
        printf("The array of random numbers is not sorted.\n");
    }
    
    
    printf("Initial array 1: ");
    print_array(arr1, ARRAY_SIZE);
    
    start = clock();
    qs_sort(arr1, 0, ARRAY_SIZE - 1);
    end = clock();
    double time_taken_qs = (end - start);
    
    if (is_sorted(arr1, ARRAY_SIZE)) {
        printf("The array is sorted after quicksort.\n");
    } else {
        printf("Error: Array not sorted after quicksort.\n");
    }
    
    printf("Array 1 after quicksort: ");
    print_array(arr1, ARRAY_SIZE);
    if (is_sorted(arr1, ARRAY_SIZE)) {
        printf("The array 1 is sorted after quicksort.\n");
    } else {
        printf("Error: Array 1 not sorted after quicksort.\n");
    }
    printf("Quicksort execution time: %f seconds\n\n\n", time_taken_qs);
    
    
    printf("Initial array 2: ");
    print_array(arr2, ARRAY_SIZE);
    
    start = clock();
    insertion_sort(arr2, ARRAY_SIZE);
    end = clock();
    double time_taken_insertion = (end - start);
    
    if (is_sorted(arr2, ARRAY_SIZE)) {
        printf("The array after insertion sort is sorted.\n");
    } else {
        printf("Error: Array is not sorted after insertion sort.\n");
    }
    
    printf("Array 2 after quicksort: ");
    print_array(arr2, ARRAY_SIZE);
    if (is_sorted(arr2, ARRAY_SIZE)) {
        printf("The array 2 is sorted after insertion sort.\n");
    } else {
        printf("Error: Array 2 not sorted after insertion sort.\n");
    }
    printf("Insertion sort execution time: %f seconds\n\n\n", time_taken_insertion);
    
    
    printf("Initial array 3: ");
    print_array(arr3, ARRAY_SIZE);
    
    start = clock();
    insertion_sort(arr3, ARRAY_SIZE);
    end = clock();
    double time_taken_classic_insertion = (end - start);
    
    if (is_sorted(arr3, ARRAY_SIZE)) {
        printf("The array is sorted after classical insertion sorting.\n");
    } else {
        printf("Error: The array is not sorted after classic insertion sort.\n");
    }
    
    
    printf("Array 3 after quicksort: ");
    print_array(arr3, ARRAY_SIZE);
    if (is_sorted(arr3, ARRAY_SIZE)) {
        printf("The array 3 is sorted after classic insertion sort.\n");
    } else {
        printf("Error: Array 3 not sorted after classic insertion sort.\n");
    }
    printf("Execution time of classic insertion sort: %f seconds\n", time_taken_classic_insertion);
    
    
    return 0;
}
